package com.subsecret.dnstester.common;

import java.util.ArrayList;
import java.util.Collections;

public abstract class DNSLogger {
    
    private int success = 0;
    private int failed = 0;
    private int count = 0;
    ArrayList<Long> delay = new ArrayList<Long>();
    
    public void replySuccess(long delay, String address) {
        this.delay.add(delay);
        success++;
        count++;
        log("Resolved host. Took " + delay / 1000000 + " ms." + " (IP: " + address + ")");
        update();
    }
    
    public abstract void log(String log);
    
    public abstract void update();
    
    public int getCount() {
        return count;
    }
    
    public int getMin() {
        if (delay.size() == 0) {
            return 0;
        }
        long min = Long.MAX_VALUE;
        for (long val : delay) {
            if (val < min) {
                min = val;
            }
        }
        return (int) (min / 1000000L);
    }
    
    public abstract void done();
    
    public void clear() {
        success = 0;
        failed = 0;
        count = 0;
        delay.clear();
        update();
    }
    
    public int getMax() {
        long max = 0;
        for (long val : delay) {
            if (val > max) {
                max = val;
            }
        }
        return (int) (max / 1000000L);
    }
    
    public int getFailed() {
        return failed;
    }
    
    public int getSucceeded() {
        return success;
    }
    
    public int getAverageDelay() {
        long total = 0;
        long size = (long) delay.size();
        for (long d : delay) {
            total += d;
        }
        return (int) (((((double) total) / ((double) size))) / 1000000L);
    }
    
    public int getMedianDelay() {
        if (delay.size() == 0) {
            return 0;
        }
        ArrayList<Long> sorted = new ArrayList<Long>();
        for (long d : delay) {
            sorted.add(d);
        }
        Collections.sort(sorted);
        int median = sorted.size() / 2;
        return (int) (sorted.get(median) / 1000000L);
        
    }
    
    public void replyFailure(long delay, String host, Exception e) {
        failed++;
        count++;
        log("Failed to resolve host. Took " + delay / 1000000 + " ms." + " (HOST: " + host + ")" + " (Exception "
                + e.getClass().getSimpleName() + ": " + e.getMessage() + ")");
    }
}
