package com.subsecret.dnstester.common;

import java.lang.reflect.Field;
import java.net.InetAddress;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import org.xbill.DNS.ExtendedResolver;
import org.xbill.DNS.SimpleResolver;

import sun.net.InetAddressCachePolicy;

public class DNSTesterTool {
    
    public static String[] DEFAULT_HOSTS = new String[] { "www.subsecret.dk", "en.wikipedia.org", "kernel.org", "www.google.com",
            "www.dr.dk", "www.bbc.co.uk", "www.tv2.dk", "www.cnn.com", "www.tomshardware.com", "www.twitter.com", "www.amd.com",
            "www.intel.com" };
    
    public static List<String> getDns() {
        List<String> nameservers = sun.net.dns.ResolverConfiguration.open().nameservers();
        return nameservers;
    }
    
    public static void main(String[] args) throws Exception {
        ExtendedResolver defaultResolver = (ExtendedResolver) org.xbill.DNS.Lookup.getDefaultResolver();
        defaultResolver.deleteResolver(defaultResolver.getResolver(1));
        defaultResolver.deleteResolver(defaultResolver.getResolver(0));
        System.out.println(defaultResolver);
        defaultResolver.addResolver(new SimpleResolver("192.168.0.1"));
        InetAddress byName = org.xbill.DNS.Address.getByName("www.subsecret.dk");
        System.out.println(byName.getHostAddress());
    }
    
    public static void main(String dns, int delay, String[] hosts, AtomicBoolean run, DNSLogger logger, int targetCount)
            throws Exception {
        org.xbill.DNS.Lookup.setDefaultResolver(new SimpleResolver(dns));
        /*
         * if (!dns.equals("SYSTEM")) { System.setProperty("sun.net.spi.nameservice.nameservers", dns);
         * System.setProperty("sun.net.spi.nameservice.provider.1", "dns,sun"); } else {
         * System.getProperties().remove("sun.net.spi.nameservice.nameservers");
         * System.getProperties().remove("sun.net.spi.nameservice.provider.1"); }
         */
        Field f = InetAddressCachePolicy.class.getDeclaredField("cachePolicy");
        f.setAccessible(true);
        f.setInt(null, InetAddressCachePolicy.NEVER);
        int count = 0;
        while (run.get()) {
            for (String host : hosts) {
                if (!run.get()) {
                    break;
                }
                if (count >= targetCount) {
                    run.set(false);
                    break;
                }
                long start = System.nanoTime();
                try {
                    InetAddress inet = org.xbill.DNS.Address.getByName(host);
                    org.xbill.DNS.Lookup.getDefaultCache(1).clearCache();

                    // InetAddress inet = InetAddress.getByName(host);
                    long end = System.nanoTime();
                    logger.replySuccess(end - start, inet.getHostAddress());
                    if (delay > 0) {
                        Thread.sleep(delay);
                    }
                    
                }
                catch (Exception e) {
                    long end = System.nanoTime();
                    logger.replyFailure(end - start, host, e);
                }
                count++;
            }
        }
        run.set(false);
        logger.done();
    }
}
