package com.subsecret.dnstester.gui;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

import com.subsecret.dnstester.common.DNSTesterTool;

public class DNSTesterGUI {
    private static JTextArea dnsservers = new JTextArea(4, 30);
    private static JTextArea log = new JTextArea(7, 30);
    private static JTextField dnsToUse = new JTextField(30);
    private static JTextField delay = new JTextField("1000");
    private static JTextField tcount = new JTextField("1000");
    private static JTextArea hosts = new JTextArea(4, 30);
    private static JButton reset = new JButton("Reset");
    private static JToggleButton start = new JToggleButton("Start");
    
    private static JLabel count = new JLabel("0");
    private static JLabel success = new JLabel("0");
    private static JLabel failed = new JLabel("0");
    private static JLabel avgDelay = new JLabel("0 ms");
    private static JLabel medianDelay = new JLabel("0 ms");
    private static JLabel minDelay = new JLabel("0 ms");
    private static JLabel maxDelay = new JLabel("0 ms");
    
    public static void main(String args[]) {
        UIManager.put("swing.boldMetal", Boolean.FALSE);
        final GUIDNSLogger logger = new GUIDNSLogger(log, avgDelay, medianDelay, minDelay, maxDelay, count, success, failed, start);
        final AtomicBoolean run = new AtomicBoolean(true);
        reset.addActionListener(new ActionListener() {
            
            @Override
            public void actionPerformed(ActionEvent e) {
                logger.clear();
            }
        });
        start.addActionListener(new ActionListener() {
            
            @Override
            public void actionPerformed(ActionEvent e) {
                if (start.isSelected()) {
                    run.set(true);
                    Runnable runner = new Runnable() {
                        
                        @Override
                        public void run() {
                            try {
                                String[] h = hosts.getText().split("\n");
                                for (int i = 0; i < h.length; i++) {
                                    h[i] = h[i].trim();
                                }
                                DNSTesterTool.main(dnsToUse.getText(), Integer.parseInt(delay.getText()), h, run, logger,
                                        Integer.parseInt(tcount.getText()));
                            }
                            catch (NumberFormatException e) {
                                // TODO Auto-generated catch block
                                e.printStackTrace();
                            }
                            catch (Exception e) {
                                // TODO Auto-generated catch block
                                e.printStackTrace();
                            }
                            
                        }
                    };
                    new Thread(runner).start();
                }
                else {
                    run.set(false);
                }
            }
            
        });
        dnsservers.setMinimumSize(dnsservers.getPreferredSize());
        dnsservers.setBorder(delay.getBorder());
        dnsservers.setEditable(false);
        for (String d : DNSTesterTool.getDns()) {
            dnsservers.append(d + "\n");
        }
        dnsToUse.setText(DNSTesterTool.getDns().get(0));
        delay.setMinimumSize(delay.getPreferredSize());
        for (String host : DNSTesterTool.DEFAULT_HOSTS) {
            hosts.append(host + "\n");
        }
        JFrame frame = new JFrame("DNS Tester (Version 1.10)");
        //frame.setResizable(false);
        frame.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        Container contentPane = frame.getContentPane();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = new Insets(2, 2, 2, 2);
        gbc.fill = GridBagConstraints.BOTH;
        
        gbc.weightx = 0;
        gbc.gridy++;
        contentPane.add(new JLabel("System DNS Server: "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(dnsservers, gbc);
        
        gbc.weightx = 0;
        gbc.gridx = 0;
        gbc.gridy++;
        contentPane.add(new JLabel("DNS Server To Test: "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(dnsToUse, gbc);
        
        gbc.weightx = 0;
        gbc.gridx = 0;
        gbc.gridy++;
        contentPane.add(new JLabel("Delay (ms.): "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(delay, gbc);
        
        gbc.weightx = 0;
        gbc.gridx = 0;
        gbc.gridy++;
        contentPane.add(new JLabel("Count: "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(tcount, gbc);
        
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.gridwidth = 2;
        contentPane.add(new JLabel("Hosts to lookup (one on each line):"), gbc);
        gbc.gridy++;
        JScrollPane hpane = new JScrollPane(hosts);
        contentPane.add(hpane, gbc);
        
        gbc.gridwidth = 1;
        gbc.weightx = 0;
        gbc.gridx = 0;
        gbc.gridy++;
        contentPane.add(reset, gbc);
        gbc.weightx = 0;
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(start, gbc);
        
        gbc.gridwidth = 2;
        gbc.gridx = 0;
        gbc.gridy++;
        contentPane.add(new JSeparator(SwingConstants.HORIZONTAL), gbc);
        gbc.gridwidth = 1;
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Count:"), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(count, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Success:"), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(success, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Failed:"), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(failed, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Avg. Delay:"), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(avgDelay, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Median Delay: "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(medianDelay, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Min Delay: "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(minDelay, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx = 0;
        contentPane.add(new JLabel("Max Delay: "), gbc);
        gbc.gridx = 1;
        gbc.weightx = 1;
        contentPane.add(maxDelay, gbc);
        
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.gridwidth = 2;
        contentPane.add(new JLabel("Log:"), gbc);
        gbc.gridy++;
        gbc.weighty = 1;
        JScrollPane pane = new JScrollPane(log);
        contentPane.add(pane, gbc);
        gbc.weighty = 0;
        gbc.gridy++;
        contentPane.add(new JLabel("Copyright SubSecret Software (www.subsecret.dk)"), gbc);
        gbc.gridy++;
        contentPane.add(new JLabel("This software is released under the GPL"), gbc);
        frame.pack();
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
        
    }
}
